#include <iostream>

#include <getopt.h>

#include "Game.h"
#include "gaming-core/rendering/Shader.h"
#include "gaming-core/rendering/Window.h"
#include "gaming-core/rendering/WindowOptions.h"
#include "gaming-core/utils/Clock.h"
#include "gaming-core/wrapper/GL.h"
#include "rendering/Renderer.h"

bool parseArgs(int argAmount, char* const* args, WindowOptions& options) {
    while(true) {
        switch(getopt(argAmount, args, "fv")) {
            case '?': return true;
            case 'f': options.fullscreen = true; break;
            case 'v': options.vsync = true; break;
            case -1: return false;
        }
    }
}

int main(int argAmount, char* const* args) {
    Size size(800, 480);
    WindowOptions options(3, 0, size, true, "Pigine");
    if(parseArgs(argAmount, args, options)) {
        return 0;
    }

    TextInput* input = nullptr;
    Window w(input, options);
    if(w.getError().has()) {
        std::cout << w.getError().message << '\n';
        return 0;
    }

    Shader shader("resources/shader/vertex.vs", "resources/shader/fragment.fs");
    if(shader.hasError()) {
        return 0;
    }

    Controller controller(w.getButtons());
    static Game game(controller, w.getFrameClock(), w.getTickClock(), size);

    struct GameBase {
        Window& window;
        Game& game;
        Shader& shader;
        Renderer renderer;

        GameBase(Window& w, Game& g, Shader& s)
            : window(w), game(g), shader(s), renderer(s) {
        }

        void tick() {
            game.tick();
        }

        void render(float lag) {
            GL::clear();
            const Size& size = window.getSize();
            GL::setViewport(size.width, size.height);

            Matrix view;
            view.scale(Vector3(2.0f / size.width, -2.0f / size.height, 1.0f));
            view.translate(Vector3(-1.0f, 1.0f, 0.0f));
            shader.setMatrix("view", view.getValues());

            game.render(lag, renderer);
            GL::printError("GL-Error");
        }

        bool isRunning() const {
            return game.isRunning();
        }
    };
    GameBase base(w, game, shader);
    w.run(base, 10'000'000);
    return 0;
}